from django.shortcuts import render, get_object_or_404, redirect
from django.template import Template, Context
from django.http import HttpResponse
from django.urls import reverse
from django.views.generic.list import ListView
from django.views.generic.detail import DetailView
import django.contrib.auth as auth
from . import models

class TemplateListView(ListView):
    model = models.KubeTemplate

    def get_queryset(self):
        if self.request.user.is_authenticated:
            return models.KubeTemplate.objects.all()
        queryset = models.KubeTemplate.objects.filter(listed=True)
        return queryset

class TemplateDetailView(DetailView):
    model = models.KubeTemplate
    slug_field = "name"
    slug_url_kwarg = "name"

def template_yaml(request, name):
    kubetemplate = get_object_or_404(models.KubeTemplate, name=name)

    template = Template(kubetemplate.template)

    context = {}
    missing = []
    for param in kubetemplate.parameters.all():
        key = param.name
        if key not in request.GET:
            if param.is_required:
                missing.append(key)
                continue
            else:
                value = param.default
        else:
            value = request.GET.get(key)

        if not value and not param.is_required:
            # Suppress optional empty values
            continue

        try:
            context[key] = param.eval(value)
        except ValueError as e:
            print(e)
            return render(
                request,
                "kubetemplates/invalid_param.txt",
                context={"parameter_name": key},
                status=400,
                content_type="text/plain",
            )
            
    if missing:
        return render(
            request,
            "kubetemplates/missing_param.txt",
            context={"missing": missing},
            status=400,
            content_type="text/plain",
        )
    
    context = Context(context)
    response = HttpResponse(template.render(context))
    response["Content-type"] = "text/plain"
    return response

def logout(request):
    if request.user.is_authenticated:
        auth.logout(request)

    return redirect(reverse("template-list"))
